=begin 

Copyright 2012 Tim Warner

Description: A Sketchup plug-in that creates the specification for a lamella roof covering a rectangular space. 

Inputs:      The required span of the roof, and its height above the edges. 
             The number of lamellas in each arch.
             The angle each arch makes with a transverse section (so-called "skew angle").
             The cross-section of the lamella, excluding the curved portion.
             Specifications of the bolt connection for the lamellas.

Outputs:     A Sketchup model of the roof
             Text output (written to the Ruby console):
                The geometry of the lamella, including the position and orientation of bolts, and cutting / drilling information.
                The geometry of the supporting beams, at the edges of the roof.
                The geometry of the braces between the lamellas at the edges of the roof.
                The geometry of the gable

Reference:   Tim Warner, "The Geometry of the Lamella Roof" (available from lamella@timwarner.ca)

Limitations: There is no error-checking on inputs. For roofs higher than a semicircle in cross-section the geometry of beams and braces
             is incorrect.
             The gable ends of the roof will be specific to the space: that is, the length of the gable lamellas is not determined here.
             The model does not include decking, insulation etc., that would make a complete roof.
             The model is a drawing tool only -- it knows nothing about engineering and structural soundness.

Known bugs:  The use of the intersect_with method fails very occasionally, leaving the lamellas lacking a face or two.  This can be remedied manually by 
             editing the component to add the (obvious) missing edge.
             
Support:     Email lamella@timwarner.ca

          
Release Notes:

30 May 2012  Release 0.1
25 July 2012  Release 0.2
  Fixed issues regarding interference of the plugin with other plugins per community comments
  Restructured code for greater modularity
  Fixed error in the way the tops of lamellas were computed
1 August 2012  Release 0.3
  Added the functionality to model a diagrid roof
  Added some error-checking on input, and a spinner for number of bolts
  
  -------------------------------------------------------------------------
  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  -------------------------------------------------------------------------
=end

require 'sketchup.rb'

module TimWarner
  
def self.lamella_roof()

  model=Sketchup.active_model
  ents = model.entities
  opts = model.rendering_options
  opts["DrawLineEnds"] = false
  selection = model.selection
  
# Hide all the active drawing elements in the model.

  pre_existing_entities = model.active_entities.select{|e| e.is_a?(Sketchup::Drawingelement)}
  pre_existing_entities.each{|e| e.hidden = true}

  diagrid = false
  prompts = ["Type of roof"]
  values = ["Traditional"]
  results = UI.inputbox(prompts, values,["Traditional|Diagrid"],"Type of Roof")
  if results[0] == "Traditional"
    diagrid = false
  else
    diagrid = true
  end
    
# Get roof specifications
  specs_incomplete = true
    if diagrid
      results = []
      while specs_incomplete 
      prompts = ["Height of roof", "Span of roof", "Skew angle","Lamellas","Arches"]
      values = [1000.0.mm,4000.0.mm,45.0,5,5]  
      results = UI.inputbox(prompts, values, "Roof Settings")
      if (results[2] >= 15.0 && results[2] <= 45.0) 
        specs_incomplete = false
      else 
        UI.beep
        answer = UI.messagebox("Skew angle must be between 15 and 45",MB_OKCANCEL)
        return if answer == 2  
      end
    end
        
    else  
      prompts = ["Height of roof", "Span of roof", "Skew angle","Lamellas","Arches"]
      values = [1000.0.mm,4000.0.mm,20.0,5,5]  
      results = UI.inputbox(prompts, values, "Roof Settings")
      return nil unless results
    end
    puts "Roof Settings"
    print "Height ", results[0],"\n"
    print "Span ", results[1],"\n"
    print "Skew ", results[2],"\n"
    print "Lamellas ", results[3],"\n"
    print "Arches ", results[4],"\n"

    height = results[0].to_f
    span = results[1].to_f
    skew_angle = results[2].to_f.degrees
    lamellas = results[3].to_i
    number_of_arches = results[4].to_i
    roof_radius = (4*height**2 + span**2) / (8*height)

# If the section higher than a semi-circle the formula for theta will be different

    if height <= span/2.0 
      theta = Math.asin(span / (2*roof_radius))/lamellas
    else
      theta = (Math::PI - Math.asin(span / (2*roof_radius)))/lamellas
    end

    initial_lamella_length = 2.0 * roof_radius * Math.sin(theta) / Math.cos(skew_angle)

#  Get lamella specifications
    if diagrid 
      prompts = ["Thickness", "Width"]
      values = [40.0.mm,150.0.mm]
      results = UI.inputbox(prompts, values, "Lamella dimensions")
      return nil unless results
      thickness = results[0]
      width = results[1]
      number_of_bolts = 0
      bolt_diameter = 0.0
      adjusted_lamella_length = initial_lamella_length - (thickness / Math.sin(2.0*skew_angle)) - (thickness / Math.tan(2.0*skew_angle))
    else
      prompts = ["Thickness", "Width","Bolts", "Bolt diameter"]
      values = [40.0.mm,150.0.mm,2,10.0.mm]
      results = UI.inputbox(prompts, values, ["","","1|2",""],"Lamella dimensions")
      return nil unless results   
      thickness = results[0]
      width = results[1]
      number_of_bolts = results[2]
      bolt_diameter = results[3]
      adjusted_lamella_length = initial_lamella_length + 
                                thickness * (1 + 1 / Math.cos(2*skew_angle)) * Math.tan(skew_angle) + 
                                bolt_diameter*Math.tan(2*skew_angle) * Math.tan(skew_angle) + bolt_diameter
    end
    puts
    puts "Lamella Settings"
    print "Thickness ", thickness, "\n"
    print "Width ", width, "\n"
    print "Bolts ", number_of_bolts, "\n"
    print "Bolt diameter ", bolt_diameter, "\n"
    

# Compute hba = horizontal bevel angle and vba = vertical bevel angle

    hba_num = Math.sin(skew_angle)*Math.cos(skew_angle)*(1 + Math.cos(theta))
    hba_denom = 1 - (1 + Math.cos(theta)) * (Math.sin(skew_angle))**2  
    tan_hba = hba_num / hba_denom
    hba = Math.atan(tan_hba)

    vba_num = Math.tan(theta)
    vba_denom = Math.cos(skew_angle) * (1 + 1 / Math.cos(theta))
    tan_vba = vba_num / vba_denom
    vba = Math.atan(tan_vba)

    adjusted_theta = Math.asin(adjusted_lamella_length / (2*roof_radius))
    delta_width = roof_radius * (1 - Math.cos(adjusted_theta))

    lamella_blank_length = adjusted_lamella_length + 2 * thickness / tan_hba + 2 * (delta_width + width/2) * tan_vba
  
    top_height = width/2 + delta_width  # figure out how much wider the lamella has to be to accommodate curvature
    length = adjusted_lamella_length
    
    hl = length / 2.0
    ht = thickness / 2.0
    hw = width / 2.0

    model.start_operation('lamella')

#  ------------------------ Make base lamella blank ---------------------------------------------

  # Create the center of the lamella, push/pull to width of lamella

    pts = []
    pts[0] = [-(hl + thickness / tan_hba),-ht, 0] 
    pts[1] = [-hl, ht, 0]
    pts[2] = [(hl + thickness / tan_hba),ht, 0]
    pts[3] = [hl,-ht, 0]

    lamella_blank = ents.add_group()
    base  = lamella_blank.entities.add_face(pts)

  # Make the top

    base.pushpull(-top_height,true)

  # Get the edges of the top face

    top = lamella_blank.entities.select{|e| e.is_a?(Sketchup::Face) && e.plane[3] == -top_height }[0]

    top_left_edge = top.edges.select{|e| e.start.position.x < 0 && e.end.position.x < 0}[0]
    top_right_edge = top.edges.select{|e| e.start.position.x > 0 && e.end.position.x > 0}[0]

  # Adjust top for vertical bevel

    delta = top_height * tan_vba
    movetop = Geom::Vector3d.new(-delta,0,0)
    trans = Geom::Transformation.new(movetop)
    lamella_blank.entities.transform_entities(trans,top_left_edge)
    movetop = Geom::Vector3d.new(delta,0,0)
    trans = Geom::Transformation.new(movetop)
    lamella_blank.entities.transform_entities(trans,top_right_edge)
  
  # Make the bottom

    base.pushpull(-hw,false)

  # Get the edges of the bottom face

    bottom = lamella_blank.entities.select{|e| e.is_a?(Sketchup::Edge) && e.start.position.z == -hw }
    bottom_left_edge = bottom.select{|e| e.start.position.x < 0 && e.end.position.x < 0}[0]
    bottom_right_edge = bottom.select{|e| e.start.position.x > 0 && e.end.position.x > 0}[0]

  # Adjust bottom for vertical bevel

    delta = hw * tan_vba
    movebottom = Geom::Vector3d.new(delta,0,0)
    trans = Geom::Transformation.new(movebottom)
    lamella_blank.entities.transform_entities(trans,bottom_left_edge)
    movebottom = Geom::Vector3d.new(-delta,0,0)
    trans = Geom::Transformation.new(movebottom)
    lamella_blank.entities.transform_entities(trans,bottom_right_edge)
        
  # Remove superfluous edges used in construction; paint the lamella

    mid_edges = []
    lamella_blank.entities.each{|e|
      if e.is_a?(Sketchup::Edge)
        mid_edges << e if (e.start.position.z == 0 and e.end.position.z == 0)
      end
    }
    lamella_blank.entities.erase_entities(mid_edges)
    lamella_blank.entities.each{|e| e.material = "BurlyWood" if !e.deleted? and e.is_a?(Sketchup::Face)}

 # Write lamella blank specifications

    print "\nLamella blank length "
    printf("%0.02f",lamella_blank_length.to_mm)
    puts
    puts "\nLamella blank upper and lower corners"
    top_and_bottom = lamella_blank.entities.select{|e| e.is_a?(Sketchup::Face) && (e.normal == [0,0,1] or e.normal == [0,0,-1])}
    v = []
    top_and_bottom.each{|f| v << f.vertices}
    vf = v.flatten
    vf.sort{|a,b| a.position.z <=> b.position.z}
    vf.each{|vv| printf("\n%0.02f,%0.02f,%0.02f",vv.position.x.to_mm, (vv.position.y + ht).to_mm, (vv.position.z + hw).to_mm)}  
    puts
     
  # Write the compound mitre saw angles (2 options, depending on orientation of lamella blank)
    
    faces = bottom_right_edge.faces
    mitre_angle = faces[0].normal.angle_between(faces[1].normal)
    puts "\nCompound mitre saw cuts (Option A)"
    print "Horizontal angle "
    printf("%0.02f",hba.radians)
    print "\nVertical angle "
    printf("%0.02f", mitre_angle.radians)
    puts    
    edge = lamella_blank.entities.select{|e| e != nil && e.is_a?(Sketchup::Edge) && !(e.line[1].z == 0)}[0]  # Chooses the first non-horizontal edge
    faces = edge.faces
    mitre_angle = faces[0].normal.angle_between(faces[1].normal)
    puts "\nCompound mitre saw cuts (Option B)"
    print "Horizontal angle "
    printf("%0.02f",vba.radians)
    print "\nVertical angle "
    printf("%0.02f",mitre_angle.radians)
    puts
    lamella_blank.hidden = true
      
# ---------------------------- End of lamella blank construction ----------------------------------

# Set up the different kinds of lamella

    lamella = lamella_blank.copy              # Whole lamella
    lamella.make_unique
 
    gable_lamella = lamella.copy              # Gable lamellas are different
    gable_lamella.make_unique
    gable_lamella.hidden = true
    
    half_lamella = lamella.copy               # Half lamellas at sills are different
    half_lamella.make_unique
    half_lamella.hidden = true
  
# ------------------------------ Create whole lamella component ------------------------------------------

    lamella = maketop(lamella,roof_radius,theta,skew_angle,width,top_height,thickness,length)  # Make the curved top surface of the lamella, and return the lamella component
    lamella.definition.name = "Lamella"
    lamella.hidden = true

#  Write the location of the vertices in the face of the lamella

   lw2 = lamella.definition.bounds.width / 2.0
   puts "\n(x,y) coordinates of vertices on the face of the lamella"
   vertices = []
   lamella.definition.entities.each{|e| 
     if ( !e.deleted? && e.is_a?(Sketchup::Edge) )
       vertices << e.start
       vertices << e.end
     end
   }
   vertices.uniq!
   back_vertices = vertices.select{|v| v.position.y == 0}
   back_vertices.sort!{|a,b| a.position.x <=> b.position.x}
   back_vertices.each{|v| printf("%0.02f, %0.02f \n",(v.position.x - lw2).to_mm,v.position.z.to_mm)}
   puts
   puts "Exact description of elliptical upper edge"
   roof_center_x = (thickness / 2.0) * Math.tan(skew_angle)
   roof_center_y = - roof_radius*Math.cos(theta) - width / 2.0
   printf("Center of roof in plane of front of lamella: x = %0.02f, y = %0.02f\n",roof_center_x.to_mm,roof_center_y.to_mm)
   radius = roof_radius*Math.cos(theta) + top_height - 0.125 # Reduce radius by 1/8" to ensure continuous upper curve
   ellipse_a = radius / Math.cos(skew_angle)
   ellipse_b = radius
   printf("Ellipse half-axes: major = %0.02f, minor = %0.02f \n", ellipse_a.to_mm, ellipse_b.to_mm)

# ------------------------------ Now make the extra lamellas ---------------------------------------

# The dimensions of the reduced half- and gable lamellas are obtained by inspecting a node.
# First set up a couple of faces for the front and back, and then see where they intersect
# when they are rotated into the right position
  
    front = lamella_blank.entities.select{|e| e.is_a?(Sketchup::Face) && e.normal == [0,1,0]}[0]
    back = lamella_blank.entities.select{|e| e.is_a?(Sketchup::Face) && e.normal == [0,-1,0]}[0]

    fe = front.edges.select{|e| e.start.position.z == e.end.position.z} # top and bottom edges of front
    fv = front.vertices
    be = back.edges.select{|e| e.start.position.z == e.end.position.z}  # top and bottom edges of back
    bv = back.vertices

# Build the half-lamella component

# First figure out where the half-lamella at the sill will be chopped off, i.e., define its end-face
   planes = make_node(diagrid,fv,bv,roof_radius,skew_angle,theta,thickness,bolt_diameter)
   left_front_plane = planes[0]     
   left_back_plane = planes[1]

   front_line = Geom.intersect_plane_plane(left_back_plane,front.plane)
   pts = []
   fe.each{|e| 
     pts << Geom.intersect_line_line(front_line,e.line)
   }  # Find where line intersects edges
   pts.sort!{|a,b| a.z <=> b.z}
   pt0 = pts[0]
   pt1 = pts[1]
   pt2 = Geom::Point3d.new(pt1.x+thickness*Math.tan(skew_angle),-thickness/2,top_height)
   pt3 = Geom::Point3d.new(pt0.x+thickness*Math.tan(skew_angle),-thickness/2,-width/2)
   hents = half_lamella.entities
   end_face = hents.add_face(pt0,pt1,pt2,pt3)
   hents.each{|e| e.material = "BurlyWood" if !e.deleted? and e.is_a?(Sketchup::Face)}     

# Calcuate here the top of the end face (based on intersection of left edge and elliptical top)
# Standard algebra for intersection of line and ellipse, taking the first root of the quadratic
# This sets up a copy of the half-lamella end-face, which is used later to position the beam

   slope = (pt1.z - pt0.z)/(pt1.x - pt0.x)
   constant = pt0.z - pt0.x*slope
   roof_center_y = roof_center_y + hw
   quad_a = (slope*ellipse_a)**2 + ellipse_b**2
   quad_b = 2*(slope*constant*ellipse_a**2 - slope*roof_center_y*ellipse_a**2 - roof_center_x*ellipse_b**2)
   quad_c = (ellipse_b*roof_center_x)**2 + (ellipse_a*(constant - roof_center_y))**2 - (ellipse_a * ellipse_b)**2
   root_1 = (-quad_b + (quad_b**2 - 4*quad_a*quad_c)**0.5)/(2*quad_a)
   pt1.x = root_1
   pt1.z = slope*root_1 + constant
   pt2 = Geom::Point3d.new(pt1.x+thickness*Math.tan(skew_angle),-thickness/2,pt1.z)
   pt3 = Geom::Point3d.new(pt0.x+thickness*Math.tan(skew_angle),-thickness/2,-width/2)
   end_face_group = ents.add_group()
   ee = end_face_group.entities
   end_face_copy = ee.add_face(pt0,pt1,pt2,pt3)
   move_end = Geom::Vector3d.new(-pts[0].x,ht,hw)
   move_end_face = Geom::Transformation.new(move_end)
   ee.transform_entities(move_end_face,end_face_copy)
   end_face_copy.hidden = true

   angle_of_end = end_face.normal.angle_between(front.normal)  # Need to figure out angle between end face and face of lamella for later
   end_face_edges = end_face.edges
   write_half_lamella_specs(end_face,ht,hw,angle_of_end) 
   all_edges = []
   hents.each{|h|
     all_edges << h if !h.deleted? and h.is_a?(Sketchup::Edge)
   }

   edges_less_end_face_edges = all_edges - end_face_edges
   edges_to_erase = []
   edges_less_end_face_edges.each{ |h|
         if h.end.position.z == -width/2
           edges_to_erase << h  if h.start.position.y > 0 and (h.start.position.x < pt0.x) || (h.end.position.x < pt0.x)
           edges_to_erase << h  if h.start.position.y < 0 and (h.start.position.x < pt3.x) || (h.end.position.x < pt3.x)
         else
           edges_to_erase << h if h.end.position.y > 0 and (h.start.position.x < pt1.x) || (h.end.position.x < pt1.x)
           edges_to_erase << h if h.end.position.y < 0 and (h.start.position.x < pt2.x) || (h.end.position.x < pt2.x)
         end
   }
# Chop off the extra half

   hents.erase_entities(edges_to_erase)
   half_lamella.hidden = false
   half_lamella = maketop(half_lamella,roof_radius,theta,skew_angle,width,top_height,thickness,length)  # Make the top surface and create component
   half_lamella.definition.name = "Half lamella"

   half_lamella.hidden = true

# Build the gable lamella component
# First figure out where the end face is

    back_line = Geom.intersect_plane_plane(left_front_plane,back.plane)
    pts = []
    be.each{|e|
     pts << Geom.intersect_line_line(back_line,e.line)
    }  # Find where line intersects back edges
    pts.sort!{|a,b| a.z <=> b.z}
    pt0 = pts[0]
    pt1 = pts[1] 
    front_line = Geom.intersect_plane_plane(left_front_plane,front.plane)   
    pts = []
    fe.each{|e| 
     pts << Geom.intersect_line_line(front_line,e.line)  # Find where line intersects front edges
    }
    pts.sort!{|a,b| a.z <=> b.z}
    pt3 = pts[0]
    pt2 = pts[1]
    gents = gable_lamella.entities
    gable_end_face = gents.add_face(pt0,pt1,pt2,pt3)
    write_gable_lamella_specs(gable_end_face,ht,hw)

# Chop off the end

    ents_to_erase = []
      gents.each{|h|
       if h.deleted?
       else
         if h.is_a?(Sketchup::Edge)
           ents_to_erase << h if (h.start.position.x > pt2.x) || (h.end.position.x > pt2.x)
         end
       end
      }
    gents.erase_entities(ents_to_erase)
    gable_lamella.hidden = false     
    gable_lamella = maketop(gable_lamella,roof_radius,theta,skew_angle,width,top_height,thickness,length)  # Make the top surface and make the gable lamella component
    gable_lamella.definition.name = "Gable lamella"
    gable_lamella.hidden = true


# ------------------------------- End of extra lamella construction ----------------------------------------
#
# Now draw the arches
#
# Set up lamella groups

  left_lamellas = ents.add_group()          # Left whole lamellas
  left_lamellas.name = "Left lamellas"
  right_lamellas = ents.add_group()         # Right whole lamellas
  right_lamellas.name = "Right lamellas"    
  gable_lamellas = ents.add_group()         # Right lamellas at gables
  gable_lamellas.name = "Gable lamellas"
  half_lamellas = ents.add_group()          # Right lamellas at sills
  half_lamellas.name = "Half lamellas"

# Set up transformations

  lcw = lamella.definition.bounds.width
  hcw = half_lamella.definition.bounds.width
  whole_lamella_shift = Geom::Vector3d.new(-lcw/2.0,-ht,-hw)
  half_lamella_shift = Geom::Vector3d.new((lcw/2.0 - hcw),-ht,-hw)

  center_lamella = Geom::Transformation.new(whole_lamella_shift)
  center_half_lamella = Geom::Transformation.new(half_lamella_shift)

  rotate_left = Geom::Transformation.new([0,0,0],[0,0,1], skew_angle)
  rotate_right = Geom::Transformation.new([0,0,0],[0,0,1], -skew_angle)
  roof_axis = -roof_radius * Math.cos(theta)
  reflect = Geom::Transformation.scaling(1,-1,1)
  if lamellas % 2 == 0     #  Number of lamellas is even

# Add whole left lamellas
    n = 1
    while n <= lamellas / 2
      angle = (2*n-1)*theta
      rotate_left_vert = Geom::Transformation.new [0,0,roof_axis],[0,1,0],angle
      rotate_right_vert = Geom::Transformation.new [0,0,roof_axis],[0,1,0],-angle
      left_lamella_instance = left_lamellas.entities.add_instance(lamella.definition,rotate_left_vert * rotate_left * reflect * center_lamella)
      left_lamella_instance.name = "Left #{lamellas / 2 - n}"
      left_lamella_instance = left_lamellas.entities.add_instance(lamella.definition,rotate_right_vert * rotate_left * reflect * center_lamella)
      left_lamella_instance.name = "Left #{lamellas / 2 + n - 1}"
      n +=1
    end

# Add whole right lamellas and gable lamellas (a whole right lamella is a reflected left lamella)

    n = 1
    while n <= (lamellas/ 2)-1
      angle = (2*n)*theta
      rotate_left_vert = Geom::Transformation.new([0,0,roof_axis],[0,1,0],angle)
      rotate_right_vert = Geom::Transformation.new([0,0,roof_axis],[0,1,0],-angle)
      right_lamella_instance = right_lamellas.entities.add_instance(lamella.definition,rotate_left_vert * rotate_right * center_lamella)
      right_lamella_instance.name = "Right #{lamellas / 2 - n}"
      new_gable_lamella = gable_lamellas.entities.add_instance(gable_lamella.definition,rotate_left_vert * rotate_right * center_lamella)
      new_gable_lamella.name = "Gable #{lamellas / 2 - n}"
      right_lamella_instance = right_lamellas.entities.add_instance(lamella.definition,rotate_right_vert * rotate_right * center_lamella)
      right_lamella_instance.name = "Right #{lamellas / 2 + n}"
      new_gable_lamella = gable_lamellas.entities.add_instance(gable_lamella.definition,rotate_right_vert * rotate_right * center_lamella)
      new_gable_lamella.name = "Gable #{lamellas / 2 + n}"
      n +=1
    end

# Add single horizontal right and gable lamellas
    top_right_lamella = right_lamellas.entities.add_instance(lamella.definition,rotate_right * center_lamella)
    top_right_lamella.name = "Top right lamella"
    new_gable_lamella = gable_lamellas.entities.add_instance(gable_lamella.definition,rotate_right * center_lamella)
    new_gable_lamella.name = "Top gable lamella"

else  # Number of lamellas is odd

# Add whole left lamellas
    n = 1
    while n <= (lamellas - 1) / 2
      angle = 2*n*theta
      rotate_left_vert = Geom::Transformation.new [0,0,roof_axis],[0,1,0],angle
      rotate_right_vert = Geom::Transformation.new [0,0,roof_axis],[0,1,0],-angle
      left_lamella_instance = left_lamellas.entities.add_instance(lamella.definition,rotate_left_vert * rotate_left * reflect * center_lamella)
      left_lamella_instance.name = "Left #{lamellas / 2 - n}"
      left_lamella_instance = left_lamellas.entities.add_instance(lamella.definition,rotate_right_vert * rotate_left * reflect * center_lamella)
      left_lamella_instance.name = "Left #{lamellas / 2 + n - 1}"
      n +=1
    end

# Add single horizontal left lamella

    top_left_lamella = left_lamellas.entities.add_instance(lamella.definition,rotate_left * reflect * center_lamella)
    top_left_lamella.name = "Top right lamella"

# Add whole right lamellas and gable lamellas
    n = 1
    while n <= (lamellas-1) / 2
      angle = (2*n - 1)*theta
      rotate_left_vert = Geom::Transformation.new([0,0,roof_axis],[0,1,0],angle)
      rotate_right_vert = Geom::Transformation.new([0,0,roof_axis],[0,1,0],-angle)
      right_lamella_instance = right_lamellas.entities.add_instance(lamella.definition,rotate_left_vert * rotate_right * center_lamella)
      right_lamella_instance.name = "Right #{lamellas / 2 - n}"
      new_gable_lamella = gable_lamellas.entities.add_instance(gable_lamella.definition,rotate_left_vert * rotate_right * center_lamella)
      new_gable_lamella.name = "Gable #{lamellas / 2 - n}"
      right_lamella_instance = right_lamellas.entities.add_instance(lamella.definition,rotate_right_vert * rotate_right * center_lamella)
      right_lamella_instance.name = "Right #{lamellas / 2 + n}"
      new_gable_lamella = gable_lamellas.entities.add_instance(gable_lamella.definition,rotate_right_vert * rotate_right * center_lamella)
      new_gable_lamella.name = "Gable #{lamellas / 2 + n}"
      n +=1
    end
  end  # Odd or even number of lamellas

# Add gable lamella at sill
    rotate_right_vert = Geom::Transformation.new([0,0,roof_axis],[0,1,0],lamellas * theta)
    new_gable_lamella = gable_lamellas.entities.add_instance(gable_lamella.definition,rotate_right_vert * rotate_right * center_lamella)
    new_gable_lamella.name = "Gable #{lamellas / 2 + n}"

# Add half lamellas  at sill
    angle = lamellas * theta
    trans = Geom::Transformation.new([0,0,roof_axis],[0,1,0],-angle)
    half_lamella_instance = half_lamellas.entities.add_instance(half_lamella.definition,trans * rotate_right * center_half_lamella)
    half_lamella_instance.name = "Half-lamella +"
    ents.transform_entities(trans * rotate_right * center_half_lamella,end_face_copy)  # Need to know where the end of the half-lamella is

    swing = Geom::Transformation.new([0,0,0],[0,0,1],180.degrees)
    half_lamella_instance = half_lamellas.entities.add_instance(half_lamella.definition,swing*trans * rotate_right * center_half_lamella)
    half_lamella_instance.name = "Half-lamella -"

# ------------------------- End of build the arches -----------------------------------------

# ------------------------- Begin assemble the roof -----------------------------------------

   inter_node_distance = 2.0 * roof_radius * Math.sin(theta) * Math.tan(skew_angle)
   if !diagrid 
     inter_node_distance += (thickness*(1+1/Math.cos(2*skew_angle))) / Math.cos(skew_angle) + bolt_diameter * Math.tan(2*skew_angle)/Math.cos(skew_angle)
   end
   half_ind = inter_node_distance / 2.0

# Find the half-lamellas and move them into position
   hl = half_lamellas.entities.find{|e| e.name == "Half-lamella +"}
   move_left = [0,half_ind,0]
   half_lamellas.entities.transform_entities(move_left,hl)
   ents.transform_entities(move_left,end_face_copy)

   hr = half_lamellas.entities.find{|e| e.name == "Half-lamella -"}
   move_right = [0,-half_ind,0]
   half_lamellas.entities.transform_entities(move_left,hr)

# Move the right lamellas into position 
   ents.transform_entities(move_left,right_lamellas)

# Set up the gable lamellas at the other end of the roof
   ents.transform_entities(move_right,gable_lamellas) 
   other_end_gable_lamellas = gable_lamellas.copy
   ents.transform_entities(swing,other_end_gable_lamellas)
   move_length_of_roof = [0,((number_of_arches) * inter_node_distance),0]
   ents.transform_entities(move_length_of_roof,other_end_gable_lamellas)

# Set up the repeating roof elements

# Turn the groups into components to make it easier to replicate new instances
   left_lamellas = left_lamellas.to_component
   right_lamellas = right_lamellas.to_component
   half_lamellas = half_lamellas.to_component

  # Move/copy the several elements
   inter_node_distance_vector = Geom::Vector3d.new(0,inter_node_distance,0)
   mc(left_lamellas,inter_node_distance_vector,number_of_arches)
   mc(right_lamellas,inter_node_distance_vector,number_of_arches-1)
   mc(half_lamellas,inter_node_distance_vector,number_of_arches-1)

# -------------------------- End of assemble the roof ---------------------------------------

# Set up beams at the edges of the roof

    normal = end_face_copy.normal
    beam_angle = Math::atan(normal.z / normal.x)
    print "\nBeam angle: "
    printf("%0.02f",beam_angle.radians)
    puts
    low_points = end_face_copy.vertices.sort{|a,b| a.position.y <=> b.position.y}
    beam = ents.add_group()
    end_face_y = low_points[2].position.y - inter_node_distance
    pts =[]
    pts[0] = [low_points[3].position.x,end_face_y,low_points[3].position.z]
    pts[1] = [low_points[2].position.x,end_face_y,low_points[2].position.z]
    pts[2] = [low_points[2].position.x,end_face_y,low_points[3].position.z]
    beam_end = beam.entities.add_face(pts)
    beam_end.material = "Chocolate"
    beam_end.reverse!
    beam_end.pushpull( ((number_of_arches+1)*inter_node_distance - thickness/Math.cos(skew_angle)),true)
    beam = beam.to_component
    reflect_x = Geom::Transformation.scaling(-1,1,1)
    delta = end_face_y.abs - low_points[0].position.y.abs  # A kludge to line up the beam with the braces (just to be pretty)
    move_beam = Geom::Vector3d.new(0,delta,0)
    trans = Geom::Transformation.new(move_beam)
    other_beam = ents.add_instance(beam.definition,trans * reflect_x)

# Write out the gable section data

    inner = low_points[1]
    radius = roof_radius*Math.cos(theta) + top_height - 0.125 # Reduce radius by 1/8" to ensure continuous upper curve
    inner_radius = ( (inner.position.x)**2 + (inner.position.z - roof_axis)**2 ) ** 0.5
    printf("Section at gable end: inner radius = %0.01f; outer radius = %0.01f\n",inner_radius.to_mm,radius.to_mm)

# Build the lamella brace

    bottom_edge_of_beam = [beam.bounds.corner(0),Geom::Vector3d.new(0,1,0)]
    top_edge_of_beam = [beam.bounds.corner(4),Geom::Vector3d.new(0,1,0)]
    length_of_brace = inter_node_distance - thickness / Math.cos(skew_angle)
    end_face_corners = end_face_copy.vertices
    end_face_corners.sort!{|a,b| a.position.y <=> b.position.y}
    brace_corners = end_face_corners[0,2]
    brace_corners.sort!{|a,b| a.position.z <=> b.position.z}
    pt0 = brace_corners[0].position
    pt1 = brace_corners[1].position
    pt2 = brace_corners[1].position + [0,-length_of_brace,0]
    pt3 = brace_corners[0].position + [0,-length_of_brace,0] 
    brace = ents.add_group()
    base_of_brace = brace.entities.add_face(pt0,pt1,pt2,pt3)
    base_of_brace.pushpull(thickness,true)
    top_of_brace = brace.entities.select{|b| b.is_a?(Sketchup::Face) and b.normal == base_of_brace.normal and b.plane != base_of_brace.plane}
    top_left_brace_edge = brace.entities.add_line(top_of_brace[0].vertices[0].position,top_of_brace[0].vertices[1].position)
    top_right_brace_edge = brace.entities.add_line(top_of_brace[0].vertices[2].position,top_of_brace[0].vertices[3].position)
    brace_bevel = 180 - angle_of_end.radians
    adj = thickness/Math::tan(brace_bevel.degrees)
    movetop = Geom::Vector3d.new(0,-adj,0)
    trans = Geom::Transformation.new(movetop)
    brace.entities.transform_entities(trans,top_left_brace_edge)
    movetop = Geom::Vector3d.new(0,adj,0)
    trans = Geom::Transformation.new(movetop)
    brace.entities.transform_entities(trans,top_right_brace_edge)
    brace = brace.to_component
    braces = ents.add_group()
    dummy = Geom::Transformation.new()
    braces.entities.add_instance(brace.definition,dummy)
    braces.entities.add_instance(brace.definition,swing)
    braces = braces.to_component
    mc(braces,inter_node_distance_vector,number_of_arches)

# Create a horizontal instance of a brace at the origin, to get the dimensions
    new_brace = ents.add_instance(brace.definition,dummy)
    trc = pt1
    brace_vector = Geom::Vector3d.new(-trc.x,-trc.y,-trc.z)
    move_brace = Geom::Transformation.new(brace_vector)
    brace_axis = Geom::Vector3d.new(0,1,0)
    rotate_brace = Geom::Transformation.new([0,0,0],brace_axis,-(90.degrees - beam_angle))
    brace_transformation = Geom::Transformation.new(rotate_brace*move_brace)
    brace_template = ents.add_instance(brace.definition,brace_transformation)
    bte = brace_template.explode
    vertices = bte.select{|e| e.is_a?(Sketchup::Vertex)}

# Write the brace specifications

    puts "\nBrace specifications (coordinates of corners)"
    vertices.each{|v| printf("%0.02f, %0.02f, %0.02f \n",v.position.x.to_mm,v.position.y.to_mm,v.position.z.to_mm)}

# Erase the temporary brace

    bte.each{|e| e.erase! if e.valid? && e.is_a?(Sketchup::Drawingelement)}
    print "\nBrace bevel: "
    printf("%0.02f",brace_bevel)
    puts

# If the roof structure is diagrid rather than traditional lamella construction don't figure out the bolt locations   
# Make a bolt -- a sub-component of a lamella

if !diagrid
    bolt = make_bolt(skew_angle,thickness,bolt_diameter,lamella)
    bolt.hidden = true

# Figure out where the bolts go
    lents = lamella.definition.entities
    front_plane = [0,1,0,-thickness]
    back_plane = [0,1,0,0]
    planes = bolt_holes(fv,bv,roof_radius,skew_angle,theta,thickness,bolt_diameter)  
    left_front = planes[0]
    left_back = planes[1]
    mid_plane = planes[2]

    shift = Geom::Vector3d.new(0,ht,0)
    move_planes = Geom::Transformation.new(shift)
    ents.transform_entities(move_planes,left_front)
    ents.transform_entities(move_planes,left_back)
    ents.transform_entities(move_planes,mid_plane)

    left_front_front_edge = left_front.edges.select{|p| p.start.position.y == p.end.position.y}[0]
    left_back_front_edge = left_back.edges.select{|p| p.start.position.y == p.end.position.y}[0]
    front_corner = left_front_front_edge.vertices.min{|e,f| e.position.z <=> f.position.z }
    back_corner = left_back_front_edge.vertices.min{|e,f| e.position.z <=> f.position.z }
    height_of_bolt = (front_corner.position.z + back_corner.position.z)/2.0
    whole_lamella_shift = Geom::Vector3d.new(lcw/2.0,ht,hw)
    shift_to_center = Geom::Transformation.new(whole_lamella_shift)
    rotate_left = Geom::Transformation.new([0,0,0],[0,0,1], -(Math::PI/2 - 2*skew_angle))
    move_over = Geom::Transformation.new([0,ht,0])

    if number_of_bolts == 1
      where_bolt_goes = Geom::Point3d.new([0,0,height_of_bolt + hw])
      move_up = Geom::Transformation.new(where_bolt_goes) # Move it into position
      bolt_instance = lents.add_instance(bolt.definition,move_up*shift_to_center*rotate_left)
      bolt_axis = ents.add_line([-100,0,0],[100,0,0])
      ents.transform_entities(move_up * shift_to_center * rotate_left,bolt_axis)
      front_centre_hole = Geom.intersect_line_plane(bolt_axis.line,front_plane)
      back_centre_hole = Geom.intersect_line_plane(bolt_axis.line,back_plane)
      printf("\nCentre hole position: Front %0.02f, %0.02f, %0.02f",(front_centre_hole.x-lcw/2).to_mm,front_centre_hole.y.to_mm,front_centre_hole.z.to_mm)
      printf("\nCentre hole position: Back %0.02f, %0.02f, %0.02f",(back_centre_hole.x-lcw/2).to_mm,back_centre_hole.y.to_mm,back_centre_hole.z.to_mm)
      ents.erase_entities(bolt_axis)
      bolt_axis = ents.add_line([-100,0,0],[100,0,0])
      ents.transform_entities(move_over * move_up * rotate_left,bolt_axis)
      front_end_hole = Geom.intersect_line_plane(bolt_axis.line,left_front.plane)
      back_end_hole = Geom.intersect_line_plane(bolt_axis.line,left_back.plane)
      puts "End hole positions:"
      bottom_front = left_front.edges.max{|a,b| a.length <=> b.length}
      bottom_back = left_back.edges.max{|a,b| a.length <=> b.length}
      front_z = front_end_hole.distance_to_line(bottom_front.line)
      back_z = back_end_hole.distance_to_line(bottom_back.line)
      printf("Z front and back %0.02f,%0.02f \n",front_z.to_mm,back_z.to_mm)
      printf("Z angle = %0.02f \n",(Math.atan(thickness/(front_z - back_z))).radians)
      front_x = front_end_hole.distance_to_plane(mid_plane.plane)
      back_x = back_end_hole.distance_to_plane(mid_plane.plane)
      printf("X front and back %0.02f, %0.02f \n",front_x.to_mm,back_x.to_mm)
      printf("X angle = %0.02f \n",(Math.atan(thickness/(front_x - back_x))).radians)
      ents.erase_entities(bolt_axis)
    else      
      bolt_number = 1
      while bolt_number <= 2
        puts "\nBolt #{bolt_number}"
        where_bolt_goes = Geom::Point3d.new([0,0,height_of_bolt + 2 * bolt_number * hw / 3])
        move_up = Geom::Transformation.new(where_bolt_goes)       # Move it into position
        lents.add_instance(bolt.definition,move_up*shift_to_center*rotate_left)
        bolt_axis = ents.add_line([-100,0,0],[100,0,0])
        ents.transform_entities(move_up * shift_to_center * rotate_left,bolt_axis)
        front_centre_hole = Geom.intersect_line_plane(bolt_axis.line,front_plane)
        back_centre_hole = Geom.intersect_line_plane(bolt_axis.line,back_plane)
        printf("Centre hole position: Front %0.02f, %0.02f, %0.02f",(front_centre_hole.x-lcw/2).to_mm,front_centre_hole.y.to_mm,front_centre_hole.z.to_mm)
        printf("\nCentre hole position: Back %0.02f, %0.02f, %0.02f",(back_centre_hole.x-lcw/2).to_mm,back_centre_hole.y.to_mm,back_centre_hole.z.to_mm)
        ents.erase_entities(bolt_axis)        
        bolt_axis = ents.add_line([-50,0,0],[50,0,0])
        ents.transform_entities(move_over * move_up * rotate_left,bolt_axis)
        front_end_hole = Geom.intersect_line_plane(bolt_axis.line,left_front.plane)
        back_end_hole = Geom.intersect_line_plane(bolt_axis.line,left_back.plane)
        puts "\nEnd hole position:"
        top_front = left_front.edges.max{|a,b| a.length <=> b.length}
        top_back = left_back.edges.max{|a,b| a.length <=> b.length}
        front_z = (width + delta_width) - front_end_hole.distance_to_line(top_front.line)
        back_z = (width + delta_width) - back_end_hole.distance_to_line(top_back.line)
        printf("Z front and back %0.02f,%0.02f \n",front_z.to_mm,back_z.to_mm)
        printf("Z angle = %0.02f \n",(Math.atan(thickness/(front_z - back_z))).radians)
        front_x = front_end_hole.distance_to_plane(mid_plane.plane)
        back_x = back_end_hole.distance_to_plane(mid_plane.plane)
        printf("X front and back %0.02f, %0.02f \n",front_x.to_mm,back_x.to_mm)
        printf("X angle = %0.02f \n",(Math.atan(thickness/(front_x - back_x))).radians)
        bolt_number = bolt_number + 1
        ents.erase_entities(bolt_axis)
      end # while bolts <=2
    end#if number of bolts
    ents.erase_entities(left_front,left_back,mid_plane)
end   #if diagrid
 
  model.commit_operation
  pre_existing_entities.each{|e| e.hidden = false}
end #lamella_roof 

# ---------------------- Here are the supporting methods ------------------------------
#
# Write the half-lamella specifications

def self.write_half_lamella_specs(end_face,ht,hw,angle_of_end)
  ents = Sketchup.active_model.entities   
  puts "\nEnd face of the half-lamella, relative to lamella blank"
  end_face.vertices.each{|v| printf("%0.02f, %0.02f, %0.02f \n",v.position.x.to_mm, (v.position.y + ht).to_mm, (v.position.z + hw).to_mm)}
  puts "\nCompound mitre saw cuts"
  inclined_edge = end_face.edges.select{|e| e.start.position.z == - hw && e.end.position.z > 0}[0]
  ip = inclined_edge.start.position
  pt2 = Geom::Point3d.new([ip.x + 10,ip.y, ip.z])
  cline = ents.add_cline(ip,pt2)
  angle_between = inclined_edge.line[1].angle_between(cline.direction)
  printf("Horizontal angle %0.02f\n",180 - angle_of_end.radians)
  printf("Vertical angle %0.02f\n",angle_between.radians )
  ents.erase_entities(cline)
end

# Write the gable lamella specifications

def self.write_gable_lamella_specs(gable_end_face,ht,hw)
  puts "\nEnd face of the gable lamella, relative to the lamella blank"
  gable_end_face.vertices.each{|v| printf("%0.02f, %0.02f, %0.02f \n",v.position.x.to_mm, (v.position.y + ht).to_mm, (v.position.z + hw).to_mm)}
  puts     
end

# Define a method to replicate a component: borrowed from Martin Rinehart's Sketchtalk

def self.mc(component,distance,number_of_copies)
  ents = Sketchup.active_model.entities
  defi = component.definition
  trans = component.transformation
  pt = trans.origin
  for i in (1..number_of_copies)
    pt += distance
    trans = Geom::Transformation.new(pt)
    ents.add_instance(defi,trans)
  end
end

=begin
  maketop is a method to construct the upper curve of a lamella. This will be applied to the three component lamellas separately. The
  reason for is that it's too tricky to build a single lamella component and then modify it, because of the segmented geometry of the 
  upper surface. 
=end

def self.maketop(roof_component,roof_radius,theta,skew_angle,width,top_height,thickness,length)

  ents = Sketchup.active_model.entities

  cylinder_length = length
  offset = cylinder_length / 2.0
  center = Geom::Point3d.new(0,-offset,-roof_radius*Math.cos(theta))
  normal = Geom::Vector3d.new(0,1,0)
  radius = roof_radius*Math.cos(theta) + top_height - 0.125 # Reduce radius by 1/8" to ensure continuous upper curve
  numsides = 400
  circle = ents.add_group()
  ngon = circle.entities.add_ngon(center,normal,radius,numsides)
  face = circle.entities.add_face(ngon)
  rotate = Geom::Transformation.new([0,0,0],[0,0,1],skew_angle)
  ents.transform_entities(rotate,circle)
  face.pushpull(cylinder_length.inch,false)
  circle.entities.each{|e| e.material = "BurlyWood" if !e.deleted? and e.is_a?(Sketchup::Face)}
  front_plane = [-Math::sin(skew_angle),Math::cos(skew_angle),0,-offset ]
  back_plane = [Math::sin(skew_angle),-Math::cos(skew_angle),0,-offset ]
  dummy = Geom::Transformation.new()
  roof_component.entities.intersect_with(true,dummy,roof_component.entities,dummy,false,circle)

  #  Remove edges above the curves in lamella

  edges_above_curves = []
  roof_component.entities.each{|e| 
    if e.is_a?(Sketchup::Edge) 
      edges_above_curves << e if (e.start.position.z == top_height || e.end.position.z == top_height)
    end
      }
  roof_component.entities.erase_entities(edges_above_curves)

  roof_component = roof_component.explode
  circle = circle.explode
  circle_edges = circle.select{|c| c.is_a?(Sketchup::Drawingelement) && c.is_a?(Sketchup::Edge)}
  ents_to_be_deleted = []
  circle_edges.each{|e| ents_to_be_deleted << e if e.start.position.distance_to_plane(front_plane) < 1 ||  # the <1 is to overcome rounding errors
                                                   e.end.position.distance_to_plane(front_plane) < 1   ||
                                                   e.start.position.distance_to_plane(back_plane) < 1  ||
                                                   e.start.position.distance_to_plane(back_plane) < 1 }

  ents.erase_entities(ents_to_be_deleted)

  ee = circle_edges.select{|e| !e.deleted? && e.is_a?(Sketchup::Edge)}[0].all_connected
  roof_component = ents.add_group(ee)
  roof_component = roof_component.to_component
  return roof_component
end

def self.make_node(diagrid,fv,bv,roof_radius,skew_angle,theta,thickness,bolt_diameter)

# Find where lamellas intersect by setting up an intersecting plane at the correct angles and position
# Returns the front plane of the interecting lamella
   ents = Sketchup.active_model.entities
# Create a dummy left lamella in the right position
   lamella_faces_dummy = ents.add_group()
   lf_ents = lamella_faces_dummy.entities
   left_front = lf_ents.add_face(fv)
   left_back = lf_ents.add_face(bv)
   roof_axis = -roof_radius * Math.cos(theta)
   rotate_vert = Geom::Transformation.new [0,0,roof_axis],[0,1,0],theta
   rotate_left = Geom::Transformation.new [0,0,0],[0,0,1], skew_angle 
   reflect = Geom::Transformation.scaling(1,-1,1)
   left_trans = rotate_vert * rotate_left * reflect
   lf_ents.transform_entities(left_trans,left_front)
   lf_ents.transform_entities(left_trans,left_back)
   inter_node_distance = 2.0 * roof_radius * Math.sin(theta) * Math.tan(skew_angle)
   if !diagrid 
     inter_node_distance += (thickness*(1+1/Math.cos(2*skew_angle))) / 
                             Math.cos(skew_angle) + bolt_diameter * Math.tan(2*skew_angle)/Math.cos(skew_angle)
   end
   half_ind = inter_node_distance / 2.0
   move_left = [0,half_ind,0]
   rotate_left = Geom::Transformation.new [0,-half_ind,0],[0,0,1], skew_angle
   left_trans = rotate_left
   lf_ents.transform_entities(left_trans,left_front)
   lf_ents.transform_entities(move_left,left_front)
   lf_ents.transform_entities(left_trans,left_back)
   lf_ents.transform_entities(move_left,left_back)   
   planes = [left_front.plane,left_back.plane]
   ents.erase_entities(lamella_faces_dummy)
   return planes
end#make_node

#  Make the bolt, washers, and nuts
  
def self.make_bolt(skew_angle,thickness,bolt_diameter,lamella)
  centre = Geom::Point3d.new(0,0,0)
  normal = Geom::Vector3d.new(-1,0,0)
  bolt_length = 2.0 * thickness + thickness / Math.cos(2*skew_angle) + bolt_diameter * Math.tan(2*skew_angle)
  centre = Geom::Point3d.new(bolt_length/2.0 - 10.mm,0,0)
  lents = lamella.definition.entities
  half_bolt = lents.add_group()
  bolt_end = half_bolt.entities.add_circle(centre,normal,bolt_diameter/2.0)
  face = half_bolt.entities.add_face(bolt_end)
  face.pushpull((bolt_length-20.mm)/2)

  centre = Geom::Point3d.new(bolt_length/2.0,0,0)
  normal = Geom::Vector3d.new(-1,0,0)
  washer = half_bolt.entities.add_circle(centre,normal,0.25 + bolt_diameter/2.0)
  face = half_bolt.entities.add_face(washer)  # washer radius is 1/4" greater than bolt
  face.pushpull(10.mm)  # washer will sink into lamella

  centre = Geom::Point3d.new(bolt_length/2.0+6.mm,0,0)
  normal = Geom::Vector3d.new(-1,0,0)
  nut = half_bolt.entities.add_ngon(centre,normal,bolt_diameter/2.0,6)
  face = half_bolt.entities.add_face(nut)
  face.pushpull(6.mm)  # 1/4" thick nut
  half_bolt = half_bolt.to_component
  
  bolt = lents.add_group()
  dummy = Geom::Transformation.new()
  bolt.entities.add_instance(half_bolt.definition,dummy) # Add the half-bolt in place
  rotate_left = Geom::Transformation.new([0,0,0],[0,0,1], -Math::PI)
  bolt.entities.add_instance(half_bolt.definition,rotate_left)  #  Add a half-bolt rotated 180 degrees
  bolt = bolt.to_component
  bolt.definition.name = "Bolt"
  lents.erase_entities(half_bolt)
  return bolt
end  #make_bolt


def self.bolt_holes(fv,bv,roof_radius,skew_angle,theta,thickness,bolt_diameter)

# Find where lamellas intersect by setting up intersecting planes at the correct angles and position
# Returns the planes of the interecting lamella (similar to make_node, but the geometry is different)

   ents = Sketchup.active_model.entities
# Create a dummy left lamella in the right position
   lamella_faces_dummy = ents.add_group()
   lamella_faces_dummy.name = "Lamella dummy"
   lf_ents = lamella_faces_dummy.entities
   left_front = lf_ents.add_face(fv)
   left_back = lf_ents.add_face(bv)
# Set up a transverse plane in the middle of the left lamella
   pt0 = Geom::Point3d.new(0,thickness/2.0,0)
   pt1 = Geom::Point3d.new(0,-thickness/2.0,0)
   pt2 = Geom::Point3d.new(0,-thickness/2.0,3)  # Height of plane is arbitrary
   pt3 = Geom::Point3d.new(0,thickness/2.0,3)
   mid_plane = lf_ents.add_face(pt0,pt1,pt2,pt3)

# Move the planes into position
   roof_axis = -roof_radius * Math.cos(theta)
   rotate_vert = Geom::Transformation.new [0,0,roof_axis],[0,1,0],theta
   rotate_left = Geom::Transformation.new [0,0,0],[0,0,1], skew_angle 
   reflect = Geom::Transformation.scaling(1,-1,1)
   left_trans = rotate_vert * rotate_left * reflect
   lf_ents.transform_entities(left_trans,left_front)
   lf_ents.transform_entities(left_trans,left_back)
   lf_ents.transform_entities(left_trans,mid_plane)
   inter_node_distance = 2.0 * roof_radius * Math.sin(theta) * Math.tan(skew_angle)
   inter_node_distance = inter_node_distance + 
                             (thickness*(1+1/Math.cos(2*skew_angle))) / 
                             Math.cos(skew_angle) + bolt_diameter * Math.tan(2*skew_angle)/Math.cos(skew_angle)
   half_ind = inter_node_distance / 2.0
   move_left = [0,half_ind,0]
   rotate_left = Geom::Transformation.new [0,-half_ind,0],[0,0,1], skew_angle
   left_trans = rotate_left
   lf_ents.transform_entities(left_trans,left_front)
   lf_ents.transform_entities(left_trans,left_back)
   lf_ents.transform_entities(left_trans,mid_plane)
   lf_ents.transform_entities(move_left,left_front)
   lf_ents.transform_entities(move_left,left_back)
   lf_ents.transform_entities(move_left,mid_plane)
   planes = [left_front,left_back,mid_plane]
   lamella_faces_dummy.hidden = true
   return planes
end#bolt_holes

# ------------------ End of supporting method definitions ----------------------------

# Put the plug-in in the menu if it isn't there already

  if( not file_loaded?("lamella_roof.rb") )

      plugins = UI.menu("Plugins")
      plugins.add_separator
      plugins.add_item("Lamella roof") { TimWarner::lamella_roof() }
  end

  file_loaded("lamella_roof.rb")

end#module TimWarner
